﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Numerics;

using SixLabors.ImageSharp;
using SixLabors.ImageSharp.PixelFormats;
using SixLabors.ImageSharp.Processing;
using SixLabors.Shapes;

using GrapeCity.Documents.Drawing;
using GrapeCity.Documents.Imaging;

using SXL = SixLabors.Shapes;
using GCI = GrapeCity.Documents.Imaging;

namespace GcImaging_vs_ImageSharp
{
    class Program
    {
        static string _outputPath;

        static void Main(string[] args)
        {
            if (UpdateOutputPath())
            {
                Console.WriteLine("300x300 image -------------------------------------------------");
                Console.WriteLine();
                Console.WriteLine("GcImaging Tests");
                SaveLogo_GcImaging(300, 120, "GcImaging_300x300_00.png", false, false);
                SaveLogo_GcImaging(300, 120, "GcImaging_300x300_01.png", false, true);
                SaveLogo_GcImaging(300, 120, "GcImaging_300x300_10.png", true, false);
                SaveLogo_GcImaging(300, 120, "GcImaging_300x300_11.png", true, true);
                Console.WriteLine("ImageSharp Test");
                SaveLogo_ImageSharp(300, 120, "ImageSharp_300x300.png");
                Console.WriteLine();
                Console.WriteLine("1000x1000 image -----------------------------------------------");
                Console.WriteLine();
                Console.WriteLine("GcImaging Tests");
                SaveLogo_GcImaging(1000, 30, "GcImaging_1000x1000_01.png", false, true);
                SaveLogo_GcImaging(1000, 30, "GcImaging_1000x1000_10.png", true, false);
                SaveLogo_GcImaging(1000, 30, "GcImaging_1000x1000_11.png", true, true);
                Console.WriteLine("ImageSharp Test");
                SaveLogo_ImageSharp(1000, 30, "ImageSharp_1000x1000.png");
                Console.WriteLine();
                Console.WriteLine("2000x2000 image -----------------------------------------------");
                Console.WriteLine();
                Console.WriteLine("GcImaging Tests");
                SaveLogo_GcImaging(2000, 10, "GcImaging_2000x2000_01.png", false, true);
                SaveLogo_GcImaging(2000, 10, "GcImaging_2000x2000_10.png", true, false);
                SaveLogo_GcImaging(2000, 10, "GcImaging_2000x2000_11.png", true, true);
                Console.WriteLine("ImageSharp Test");
                SaveLogo_ImageSharp(2000, 10, "ImageSharp_2000x2000.png");
                Console.WriteLine();
            }
        }

        public static void SaveLogo_ImageSharp(float size, int count, string fileName)
        {
            // the point are based on a 1206x1206 shape so size requires scaling from there
            float scalingFactor = size / 1206;

            Vector2 center = new Vector2(603);

            // segment whoes cetner of rotation should be 
            Vector2 segmentOffset = new Vector2(301.16968f, 301.16974f);
            SXL.IPath segment = new Polygon(new LinearLineSegment(new Vector2(230.54f, 361.0261f), new System.Numerics.Vector2(5.8641942f, 361.46031f)),
                new CubicBezierLineSegment(new Vector2(5.8641942f, 361.46031f),
                new Vector2(-11.715693f, 259.54052f),
                new Vector2(24.441609f, 158.17478f),
                new Vector2(78.26f, 97.0461f))).Translate(center - segmentOffset);

            //we need to create 6 of theses all rotated about the center point
            List<SXL.IPath> segments = new List<SXL.IPath>();
            for (int i = 0; i < 6; i++)
            {
                float angle = i * ((float)Math.PI / 3);
                SXL.IPath s = segment.Transform(Matrix3x2.CreateRotation(angle, center));
                segments.Add(s);
            }

            List<Rgba32> colors = new List<Rgba32>() {
                Rgba32.FromHex("35a849"),
                Rgba32.FromHex("fcee21"),
                Rgba32.FromHex("ed7124"),
                Rgba32.FromHex("cb202d"),
                Rgba32.FromHex("5f2c83"),
                Rgba32.FromHex("085ba7"),
            };

            Matrix3x2 scaler = Matrix3x2.CreateScale(scalingFactor, Vector2.Zero);
            var sw = new Stopwatch();

            int dimensions = (int)Math.Ceiling(size);
            using (Image<Rgba32> img = new Image<Rgba32>(dimensions, dimensions))
            {
                sw.Start();

                for (int n = 0; n < count; n++)
                {
                    img.Mutate(i => i.Fill(Rgba32.Black));
                    img.Mutate(i => i.Fill(Rgba32.FromHex("e1e1e1ff"), new EllipsePolygon(center, 600f).Transform(scaler)));
                    img.Mutate(i => i.Fill(Rgba32.White, new EllipsePolygon(center, 600f - 60).Transform(scaler)));

                    for (int s = 0; s < 6; s++)
                    {
                        img.Mutate(i => i.Fill(colors[s], segments[s].Transform(scaler)));
                    }

                    img.Mutate(i => i.Fill(new Rgba32(0, 0, 0, 170), new ComplexPolygon(new EllipsePolygon(center, 161f), new EllipsePolygon(center, 61f)).Transform(scaler)));
                }

                sw.Stop();

                img.Save(_outputPath + fileName);

                Console.WriteLine($"{count} times elapsed in {sw.ElapsedMilliseconds} ms,  Saved to {fileName}");
            }
        }

        public static void SaveLogo_GcImaging(float size, int count, string fileName, bool slowAntialiasing, bool multithreaded)
        {
            // the point are based on a 1206x1206 shape so size requires scaling from there
            float scalingFactor = size / 1206;

            PointF center = new PointF(603, 603);

            // segment whoes cetner of rotation should be 
            PointF segmentOffset = new PointF(301.16968f, 301.16974f);
            var pb = new GCI.PathBuilder();
            pb.BeginFigure(230.54f, 361.0261f);
            pb.AddLine(5.8641942f, 361.46031f);
            pb.AddCubicBezier(new PointF(-11.715693f, 259.54052f), new PointF(24.441609f, 158.17478f), new PointF(78.26f, 97.0461f));
            pb.EndFigure(true);
            var segment = pb.ToPath(FillMode.Alternate, Matrix3x2.CreateTranslation(center.X - segmentOffset.X, center.Y - segmentOffset.Y));

            //we need to create 6 of theses all rotated about the center point
            List<GraphicsPath> segments = new List<GraphicsPath>();
            for (int i = 0; i < 6; i++)
            {
                float angle = i * ((float)Math.PI / 3);
                GraphicsPath s = segment.TransformPath(Matrix3x2.CreateRotation(angle, new Vector2(center.X, center.Y)));
                segments.Add(s);
            }

            List<uint> colors = new List<uint>() {
                0xFF35a849,
                0xFFfcee21,
                0xFFed7124,
                0xFFcb202d,
                0xFF5f2c83,
                0xFF085ba7,
            };

            Matrix3x2 scaler = Matrix3x2.CreateScale(scalingFactor, Vector2.Zero);
            var sw = new Stopwatch();

            int dimensions = (int)Math.Ceiling(size);
            using (GcBitmap img = new GcBitmap(dimensions, dimensions, true))
            {
                var renderer = img.Renderer;
                renderer.SlowAntialiasing = slowAntialiasing;
                renderer.Multithreaded = multithreaded;

                sw.Start();

                for (int n = 0; n < count; n++)
                {
                    renderer.Clear(Color.Black);
                    renderer.Transform = scaler;
                    var rectF = new RectangleF(center.X - 600, center.Y - 600, 600 + 600, 600 + 600);
                    renderer.FillRegion(new Region(new EllipticFigure(rectF)), 0xFFe1e1e1);
                    rectF.Inflate(-60, -60);
                    renderer.FillRegion(new Region(new EllipticFigure(rectF)), Color.White);

                    for (int s = 0; s < 6; s++)
                    {
                        renderer.FillPath(segments[s], colors[s]);
                    }

                    rectF = new RectangleF(center.X - 161, center.Y - 161, 161 + 161, 161 + 161);
                    var reg1 = new Region(new EllipticFigure(rectF));
                    rectF = new RectangleF(center.X - 61, center.Y - 61, 61 + 61, 61 + 61);
                    var reg2 = new Region(new EllipticFigure(rectF));
                    reg1.CombineWithRegion(reg2, RegionCombineMode.Exclude, false);
                    renderer.FillRegion(reg1, Color.FromArgb(170, 0, 0, 0));
                }

                sw.Stop();

                img.SaveAsPng(_outputPath + fileName);

                Console.WriteLine($"{count} times elapsed in {sw.ElapsedMilliseconds} ms,  Saved to {fileName}  SlowAntialiasing = {slowAntialiasing}, Multithreaded = {multithreaded}");
            }
        }

        static bool UpdateOutputPath()
        {
            _outputPath = AppContext.BaseDirectory + @"..\..\..\Output";
            if (Directory.Exists(_outputPath))
                _outputPath = Directory.GetParent(_outputPath).FullName + @"\Output\";
            else
            {
                Console.WriteLine("Error: Not found the Output directory.");
                return false;
            }
            return true;
        }
    }
}
